/*
 * @(#)$Id$
 *
 * Copyright (c)2000 Regents of the University of California
 *
 */


package precache.util.graph;

import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;

/**
 *  The <code>DirectedGraphBuilder</code> description.
 *  @version $Revision$, $Date$
 *  @author Kristian Kvilekval
 */
public class DirectedGraphBuilder //extends AbstractGraphBuilder 
    implements DirectedGraph, GraphBuilder {
    
    protected GraphRep incoming;
    protected GraphRep outgoing;

    private DirectedEdgeImpl e = new DirectedEdgeImpl(null, null);

    protected Set getIncoming (Object o) {
        return incoming.getAdjacentNodes(o);
    }
    protected Set getOutgoing (Object o) {
        return outgoing.getAdjacentNodes(o);
    }

    /** Overidden GraphBuilder methods */

    public Edge createEdge (Object v1, Object v2) {
        return new DirectedEdgeImpl (v1, v2);
    }

    /** Constructors **/
    public DirectedGraphBuilder() {
        incoming = new AdjacencyListGraphRep();
        outgoing = new AdjacencyListGraphRep();
    }

    public DirectedGraphBuilder(GraphRep inEdges, GraphRep outEdges) {
        incoming = inEdges;
        outgoing = outEdges;
    }


    public void addVertex (Object o) {
        //super.addVertex (o);
        getIncoming(o);
        getOutgoing(o);
    }

    public void remVertex (Object o) {
//         super.remVertex (o);
        // Remove it from incoming edges 
        Iterator out = getOutgoing (o).iterator ();
        while (out.hasNext())
            outgoing.remEdge (e.setVertices(o, out.next()));
//         getIncoming(out.next()).remove (o);
        Iterator in = getIncoming (o).iterator ();
        while (in.hasNext())
            incoming.remEdge (e.setVertices(in.next(), o));
//             getOutgoing(in.next()).remove (o);
        outgoing.remNode (o);
        incoming.remNode (o);
    }
    public void addEdge (Object v1, Object v2 ) {
        addEdge (e.setVertices (v1, v2));
//         outgoing.addEdge (e);
//         incoming.addEdge (e);
    }
    public void remEdge (Object v1, Object v2) {
        remEdge (e.setVertices (v1, v2));
//         incoming.remEdge (v1, v2);
//         outgoing.remEdge (v2, v1);
    }
    public void remEdges (Object v) {
        //super.remEdges(v);
//         getOutgoing (v).clear();
//         getIncoming (v).clear();
    }


    public void addEdge (Edge e) {
        DirectedEdge de = (DirectedEdge)e;
        if (de.getDirection()== DirectedEdge.OUT){
            outgoing.addEdge (de);
            incoming.addEdge (de.swap());
        }else{
            incoming.addEdge (de);
            outgoing.addEdge (de.swap());
        }
    }
    public void remEdge (Edge e) {
        DirectedEdge de = (DirectedEdge)e;
        if (de.getDirection()== DirectedEdge.OUT){
            outgoing.remEdge (de);
            incoming.remEdge (de.swap());
        }else{
            incoming.remEdge (de);
            outgoing.remEdge (de.swap());
        }
    }

    /** Graph methods */
//     public int countVertices() {
//         return incoming.countNodes();
//     }
    public Set getVertices() {
        return outgoing.getNodes();
    }        
    public  Set getAdjacentVertices( Object o ) {
        return outgoing.getAdjacentNodes (o);
    }

//     public int countEdges() {
//          return incoming.getEdges().size();
//     }
    public Set getEdges() {
        return outgoing.getEdges();
    }
    public Set getEdges(Object o) {
        return outgoing.getAdjacentEdges (o);
    }


    /** DirectedGraph  */

    public Set getOutgoingAdjacentVertices( Object v ){
        return getOutgoing(v);
    }
    public Set getIncomingAdjacentVertices( Object v ){
        return getIncoming(v);
    }

    public Set getOutgoingEdges( Object v ) {
        return outgoing.getAdjacentEdges(v);
    }
    public Set getIncomingEdges( Object v ) {
        return incoming.getAdjacentEdges(v);
    }

    /** The root set contains nodes that have no 
     *  incomming edges */
    public Set getRoots () {
        // Get the set of all nodes and remove those
        // that a have non-zero number of incomming edges
        HashSet r = new HashSet();
        
        Iterator ri = incoming.getNodes().iterator();
        while (ri.hasNext()) {
            Object node = ri.next();
            if (incoming.getAdjacentEdges(node).size() == 0) 
                r.add (node);
        }
        return r;
    }


    /** Create a DirectedGraph based on the current Graph */
    public Graph getGraph() {

        //  Choose a rep for the graph based on some statistics.
//  for now just choose any one.
//         if (rep instanceof DirectedGraphNodeRep) {
//             return new DirectedGraphImpl (rep);
//         } else {
        return new DirectedGraphImpl (incoming, outgoing);
    }

};
