/*
 * @(#)$Id$
 *
 * Copyright (c)2000 Regents of the University of California
 *
 */


package precache.util.graph;

import java.util.Set;
import java.util.Iterator;
import java.util.HashMap;
import java.util.HashSet;

/**
 *  The <code>DirectedGraphImpl</code> description.
 *  @version $Revision$, $Date$
 *  @author Kristian Kvilekval
 */
public class DirectedGraphImpl 
    implements DirectedGraph{
    
    /* Private/Public Variables */

    GraphRep incoming;
    GraphRep outgoing;

    protected Set getIncoming (Object o) {
        return incoming.getAdjacentNodes(o);
    }
    protected Set getOutgoing (Object o) {
        return outgoing.getAdjacentNodes(o);
    }

    /**
     *  The default constructor
     */
    DirectedGraphImpl (GraphRep in, GraphRep out) {
        incoming = in;
        outgoing = out;
    }

    /* DirectedGraph methods */

    /** Graph methods */
//     public int countVertices() {
//         return outgoing.countNodes();
//     }
    public Set getVertices() {
        return outgoing.getNodes();
    }        
    public  Set getAdjacentVertices( Object o ) {
        return outgoing.getAdjacentNodes (o);
    }

//     public int countEdges() {
//          return outgoing.getEdges().size();
//     }
    public Set getEdges() {
        return outgoing.getEdges();
    }
    public Set getEdges(Object o) {
        return outgoing.getAdjacentEdges (o);
    }

    /** DirectedGraph  */

    public Set getOutgoingAdjacentVertices( Object v ){
        return getOutgoing(v);
    }
    public Set getIncomingAdjacentVertices( Object v ){
        return getIncoming(v);
    }

    public Set getOutgoingEdges( Object v ) {
        return outgoing.getAdjacentEdges(v);
    }
    public Set getIncomingEdges( Object v ) {
        return incoming.getAdjacentEdges(v);
    }

    /** The root set contains nodes that have no 
     *  incomming edges */
    public Set getRoots () {
        // Get the set of all nodes and remove those
        // that a have non-zero number of incomming edges
        HashSet r = new HashSet();
        
        Iterator ri = incoming.getNodes().iterator();
        while (ri.hasNext()) {
            Object node = ri.next();
            if (incoming.getAdjacentEdges(node).size() == 0) 
                r.add (node);
        }
        return r;
    }

};
