/*
 * @(#)$Id$
 *
 * Copyright (c)2000 Regents of the University of California
 *
 */


package precache.util.graph;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;


/**
 *  The <code>DirectedGraphNodeRep</code> description.
 * A representation in which the nodes themselves
 *  maintain links to other nodes.  The Object argument
 *  must support the GrahNode interface 
 *
 *  @version $Revision$, $Date$
 *  @author Kristian Kvilekval
 */

public class DirectedGraphNodeRep implements GraphRep {

    HashSet nodes;

    public DirectedGraphNodeRep() {
        nodes = new HashSet();
    }
    public void addNode (Object o) {
        nodes.add (o);
    }
    public void remNode (Object o) {
        nodes.remove (o);
    }
    public void addEdge (Object from, Object to) {
//         ((DirectedGraphNode)from).addOutEdge (to);
//         ((DirectedGraphNode)to).addInEdge (from);
        throw new UnsupportedOperationException ("addEdge");
    }
    public void remEdge (Object from, Object to){
//         ((DirectedGraphNode)from).remOutEdge (to);
//         ((DirectedGraphNode)to).remInEdge (from);
        throw new UnsupportedOperationException ("remEdge");
    }
    public boolean isNode (Object v1) {
        return nodes.contains (v1);
    }
    public boolean hasEdge (Object from, Object to){
        return ((DirectedGraphNode)from).hasOutEdge (to);
    }


    public int countNodes () {
        return nodes.size();
    }
    public Set getNodes() {
        return nodes;
    }
    public Set getAdjacentNodes (Object o){
        return ((DirectedGraphNode)o).getSuccessors();
    }


    public void addEdge (Edge e) {
        DirectedEdge de = ((DirectedEdge)e);
        if (de.getDirection() == DirectedEdge.OUT) 
            ((DirectedGraphNode)de.getSource()).addOutEdge (de.getSink());
        else
            ((DirectedGraphNode)de.getSink()).addInEdge (de.getSource());
    }
    public void remEdge (Edge e) {
        DirectedEdge de = ((DirectedEdge)e);
        if (de.getDirection() == DirectedEdge.OUT) 
            ((DirectedGraphNode)de.getSource()).remOutEdge (de.getSink());
        else
            ((DirectedGraphNode)de.getSink()).remInEdge (de.getSource());
    }
    public boolean hasEdge (Edge e) {
        DirectedEdge de = ((DirectedEdge)e);
        if (de.getDirection() == DirectedEdge.OUT) 
            return ((DirectedGraphNode)de.getSource()).hasOutEdge (de.getSink());
        else
            return ((DirectedGraphNode)de.getSink()).hasInEdge (de.getSource());
    }

    public int countEdges () {
        return 0;
    }
    public Set getEdges() {
        return null;
    }
    public Set getAdjacentEdges (Object o) {
        return null;
    }

}
