/*
 * @(#)$Id$
 *
 * Copyright (c)2000 Regents of the University of California
 *
 */

package precache.util.graph;


import java.util.Set;
import java.util.Iterator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;

/**
 *  The <code>SCCVisitor</code> description.
 *  @version $Revision$, $Date$
 *  @author Kristian Kvilekval
 */
public class SCCVisitor implements GraphVisitor{

    /* Private/Public Variables */
    protected DirectedGraph graph;
    protected GraphVisitor visitor;

    int visitNumber;
    LinkedList stack;
    HashMap n;
    HashMap l;
    HashMap SCCMap;

    private Object min( Comparable v1, Comparable v2 ) {
        if( v1.compareTo( v2 )<0 )
            return v1;
        return v2;
    }

    /**
     *  The default constructor
     */
    public SCCVisitor (DirectedGraph g, GraphVisitor v) {
        graph = g;
        visitor = v;
        
        visitNumber = 0;
        stack = new LinkedList();
        n = new HashMap();
        l = new HashMap();
        SCCMap = new HashMap();
    }

    /** Visit the nodes in the graph in Order of SCC */
    public void visit (Object v) {
        n.put( v, new Integer( visitNumber ));
        l.put( v, new Integer( visitNumber ));
        visitNumber++;
        
        stack.addLast( v );
        
        Set nextnodes = graph.getAdjacentVertices ( v );
        Iterator connected = nextnodes.iterator();
        while( connected.hasNext() ) {
            Object w = connected.next();
            
            if( n.get( w )==null ) {
                visit( w );
                l.put( v, min((Comparable)l.get(v), (Comparable)l.get(w)));
            } else if( stack.contains( w ))
                l.put( v, min((Comparable)l.get(v), (Comparable)n.get(w)));
        }
    
        if( l.get( v ).equals( n.get( v ))) {
            HashSet  SCC = new HashSet();
            Object w;
            //System.out.print ("Builing SCC: ");
            do {
                w = stack.removeLast();
                SCC.add( w );
                SCCMap.put( w, SCC );
                //System.out.print (w+" ");
            } while( !w.equals( v ));
            //System.out.println ();
            visitor.visit (SCC);
        }
    }

    private void addEdges (DirectedGraphBuilder gb) {
        // Get the edges from the original graph.  If an Edge connects
        // to different SCC then add an edge to this graph.

        Iterator edgeset = graph.getEdges().iterator();
        while (edgeset.hasNext()) {
            Edge e = (Edge)edgeset.next();
            Set SCC1 = (Set)SCCMap.get (e.getVertexA());
            Set SCC2 = (Set)SCCMap.get (e.getVertexB());
            if (SCC1 == SCC2) continue;

            gb.addEdge (SCC1, SCC2);
        }
    }

    /** Return the graph of the SCC's  */
    public static DirectedGraph getSCCGraph (DirectedGraph g, Object root) {
        final DirectedGraphBuilder gb=new DirectedGraphBuilder();

        GraphVisitor v = new GraphVisitor () {
                public void visit (Object o) {
                    Set SCC = (Set)o;
                    gb.addVertex (SCC);
                }
            };
        SCCVisitor scc = new SCCVisitor(g, v);
        scc.visit (root);
        scc.addEdges (gb);
        return (DirectedGraph)gb.getGraph();
    }

};


